#!/bin/bash
#
# Copyright 2018 Cohesity Inc.
#
# Author : Shishir Jindal(sjindal@cohesity.com)
# Common code shared by scripts configuring various flavors of Linux.

set -e
set -o nounset
set -o errexit
set -o pipefail

# Variables to store the step number at which the script stopped last time it
# was run.
completed_step_file="completed_step.txt"
completed_step=0

# Variable to indicate if the script needs to be run in the auto mode. In auto
# mode we don't prompt the user for input and we try and finish by running
# various changes automatically.
is_auto_mode=false

if [ -f $completed_step_file ]; then
  completed_step=$(cat $completed_step_file)
fi

# Function that matches a line with the given pattern ($1) in the file
# specified ($2) and replaces the line with the one specified ($3).
#
# If the line is not found it is appended at the end of the file.
f_match_replace() {
  local pattern="$1"
  local filepath="$2"
  local replacementline="$3"

  if ! grep -q "$pattern" "$filepath"; then
    # Append the replacement line at the end of the file.
    sudo echo "$replacementline" | tee -a "$filepath"
  else
    # Replace the line having pattern with the replacement line
    sudo sed -i "s/^.*${pattern}.*$/${replacementline}/" ${filepath}
  fi
}

#------------------------------------------------------------------------------

# Function to increment the completed step used during installation.
f_increment_completed_step() {
  completed_step=$((completed_step+1))
  echo $completed_step > "$completed_step_file"
}

#------------------------------------------------------------------------------

# Function to decrement the completed step used during uninstallation.
f_decrement_completed_step() {
  completed_step=$((completed_step-1))
  echo $completed_step > "$completed_step_file"
}

#------------------------------------------------------------------------------

# Function to backup a file.
f_backup() {
  local file_name="$1"
  local backup_file_name="$1.bak"

  if [ ! -f "$file_name" ]; then
    echo "$file_name not found. No backup taken."
    return 1
  fi

  echo "Backing up $file_name to $backup_file_name"
  sudo cp "$file_name" "$backup_file_name"
}

#------------------------------------------------------------------------------

# Function to prompt a message and return 0 if message read from user input
# matches expected input else return 1.
f_interactive_prompt() {
  if [ "$#" != 2 ]; then
    echo "Incorrect number of arguments. Need two arguments."
    exit 1
  fi

  local prompt_msg="$1"
  local expected_input="$2"

  # In case of auto mode, return 0, implying the expected input.
  if [ "$is_auto_mode" == true ]; then
    return 0
  fi

  # In interactive mode ask the user to provide input.
  local input
  printf "$prompt_msg\n"
  read input

  if [ "$input" == "$expected_input"  ]; then
    return 0
  fi

  return 1
}

#------------------------------------------------------------------------------

# Function to recover a file.
f_recover() {
  local file_name="$1"
  local backup_file_name="$1.bak"

  echo "We will recover $file_name from $backup_file_name"
  if [ -f "$backup_file_name" ]; then
    sudo cp "$backup_file_name" "$file_name"
    sudo rm "$backup_file_name"
  else
    echo "$backup_file_name not found."
    f_interactive_prompt "Deleting file $file_name. Please confirm (y/n)" "y"
    if [ "$?" -eq 0 ]; then
      sudo rm $file_name
    fi
  fi
}

#------------------------------------------------------------------------------

# Function to overwrite /etc/sysconfig/network file.
f_overwrite_etc_sysconfig_network() {
# The following code has to start at the begining of the line.
sudo tee /etc/sysconfig/network > /dev/null <<'EOF'
NETWORKING=yes
HOSTNAME=localhost.localdomain
EOF
}

#------------------------------------------------------------------------------

# Function to prompt and confirm while modifying /etc/sysconfig/network.
f_install_etc_sysconfig_network() {
  f_interactive_prompt "We will overwrite /etc/sysconfig/network. Are you sure? (y/n)" "y"
  if [ "$?" -eq 0 ]; then
    # In case there is no file to backup, the following command will fail and
    # we are ok with that.
    f_backup "/etc/sysconfig/network" || true

    # Write the desired content to /etc/sysconfig/network.
    f_overwrite_etc_sysconfig_network
    f_increment_completed_step
  else
    echo "exiting"
    exit 0
  fi
}

#------------------------------------------------------------------------------

# Function to uninstall changes made to /etc/sysconfig/network.
f_uninstall_etc_sysconfig_network() {
  f_recover /etc/sysconfig/network
  f_decrement_completed_step
}

#------------------------------------------------------------------------------

# Function to overwrite /etc/sysconfig/network-scripts/ifcfg-eth0.
f_overwrite_etc_sysconfig_network_script_eth0() {
# The following code has to start at the beginning of the line.
sudo tee /etc/sysconfig/network-scripts/ifcfg-eth0 > /dev/null <<'EOF'
DEVICE=eth0
ONBOOT=yes
BOOTPROTO=dhcp
PERSISTENT_DHCLIENT=yes
TYPE=Ethernet
USERCTL=no
PEERDNS=yes
IPV6INIT=no
EOF
}

#------------------------------------------------------------------------------

# Function to prompt and confirm while modifying /etc/sysconfig/network-scripts/ifcfg-eth0.
f_install_etc_sysconfig_network_script_eth0() {
  f_interactive_prompt "We will overwrite /etc/sysconfig/network-scripts/ifcfg-eth0. \nAre you sure? (y/n)" "y"
  if [ "$?" -eq 0 ]; then
    # In case there is no file to backup, the backup command will fail and we
    # are ok with that.
    f_backup "/etc/sysconfig/network-scripts/ifcfg-eth0" || true

    f_overwrite_etc_sysconfig_network_script_eth0
    f_increment_completed_step
  else
    echo "exiting"
    exit 0
  fi
}

#------------------------------------------------------------------------------

# Function to uninstall changes made to /etc/sysconfig/network-scripts/eth0.
f_uninstall_etc_sysconfig_network_script_eth0() {
  f_recover "/etc/sysconfig/network-scripts/ifcfg-eth0"
  f_decrement_completed_step
}

#------------------------------------------------------------------------------

# Function to prompt, confirm and execute
# ln -snf /dev/null /etc/udev/rules.d/75-persistent-net-generator.rules.
# Modify udev rules to avoid generating static rules for the Ethernet
# interface(s). These rules can cause problems when cloning a virtual
# machine in Microsoft Azure or Hyper-V:
f_modify_udev_rules_for_ethernet_interface() {
  f_interactive_prompt "We will overwrite /etc/udev/rules.d/75-persistent-net-generator.rules.\nAre you sure? (y/n)" "y"
  if [ "$?" -eq 0 ]; then
    # Backup will fail if the file is not found and we are ok with that.
    f_backup "/etc/udev/rules.d/75-persistent-net-generator.rules" || true
    sudo ln -snf /dev/null /etc/udev/rules.d/75-persistent-net-generator.rules
    f_increment_completed_step
  else
    echo "exiting"
    exit 0
  fi
}

#------------------------------------------------------------------------------

# Function to undo the work done by f_modify_udev_rules_for_ethernet_interface.
f_restore_udev_rules_for_ethernet_interface() {
  f_recover "/etc/udev/rules.d/75-persistent-net-generator.rules"
  f_decrement_completed_step
}

#------------------------------------------------------------------------------

# Function to start network service at boot time.
f_chkconfig_network_on() {
  # In auto mode just turn the network on.
  if [ "$is_auto_mode" == true ]; then
    sudo chkconfig network on
    f_increment_completed_step
    return
  fi

  # In non auto mode we want the user to run 'sudo chkconfig on'.
  while true; do
    echo "Please execute 'sudo chkconfig network on' and enter 'done' when done."
    read op
    if [ "$op" == "done" ]; then
      f_increment_completed_step
      return
    fi
  done
}

#------------------------------------------------------------------------------

# Function to stop network service.
f_restore_chkconfig_network_on() {
  # There is no auto mode in case network needs to be turned off.
  while true; do
    echo "Please turn off network by executing 'sudo chkconfig network off' ONLY IF NEEDED and enter 'done' when done."
    read op
    if [ "$op" == "done" ]; then
      f_decrement_completed_step
      return
    fi
  done
}

#------------------------------------------------------------------------------

# Function to uninstall networkmanager rpm.
f_uninstall_networkmanager_rpm() {
  # There is no auto mode to uninstall. The user needs to manually uninstall
  # it.
  while true; do
    echo "Please uninstall NetworkManager by executing 'sudo rpm -e --nodeps NetworkManager' and enter 'done' when done."
    read op
    if [ "$op" == "done" ]; then
      f_increment_completed_step
      return
    fi
  done
}

#------------------------------------------------------------------------------

# Function to install networkmanager rpm.
f_install_networkmanager_rpm() {
  # There is no auto mode to install. The user needs to manually install it.
  while true; do
    echo "Please install NetworkManager if needed by executing 'sudo yum install NetworkManager' and enter 'done' when done."
    read op
    if [ "$op" == "done" ]; then
      f_decrement_completed_step
      return
    fi
  done
}

#------------------------------------------------------------------------------

# Function to rebuild grub during installation.
f_rebuild_grub_during_install() {
  # Check if sshd is installed and running in the VM.
  echo "Please make sure that sshd is installed and running in the VM."
  echo "Check it by running the following command: 'sudo service sshd status'"
  while true; do
    echo "Are you sure? (y/n). If no then we will exit the script."
    read op
    if [ "$op" == "y" ]; then
      break
    elif [ "$op" == "n" ]; then
      echo "Try installing and running sshd. Exiting."
      exit 0
    fi
  done

  # In auto mode we will rebuild grub automatically.
  if [ "$is_auto_mode" == true ]; then
    sudo grub2-mkconfig -o /boot/grub2/grub.cfg
    f_increment_completed_step
    return
  fi

  while true; do
    # Rebuild the grub configuration: grub2-mkconfig -o /boot/grub2/grub.cfg.
    echo "Please execute 'sudo grub2-mkconfig -o /boot/grub2/grub.cfg' and enter 'done' when done."
    read op
    if [ "$op" == "done" ]; then
      f_increment_completed_step
      return
    fi
  done
}

#------------------------------------------------------------------------------

# Function to rebuild grub during uninstallation.
f_rebuild_grub_during_uninstall() {
  # In auto mode we will rebuild grub automatically.
  if [ "$is_auto_mode" == true ]; then
    sudo grub2-mkconfig -o /boot/grub2/grub.cfg
    f_decrement_completed_step
    return
  fi

  while true; do
    # Rebuild the grub configuration: grub2-mkconfig -o /boot/grub2/grub.cfg.
    echo "Uninstall sshd if you want to."
    echo "Please execute 'sudo grub2-mkconfig -o /boot/grub2/grub.cfg' and enter 'done' when done."
    read op
    if [ "$op" == "done" ]; then
      f_decrement_completed_step
      return
    fi
  done
}

#------------------------------------------------------------------------------

# Function to install hyperv for centos 6.3 or earlier.
f_install_kmod_hyperv_rpm() {
  # Find out if '$centos_release_str' represents centos 6.3 or lower.

  centos_release_str_arr=( $centos_release_str )
  centos_release_ver=${centos_release_str_arr[2]}

  # We are ok with string comparison. If centos release version is < 6.3, we
  # need to install microsoft hyperv rpm.
  if [[ "$centos_release_ver" > "6.3" ]]; then
    f_increment_completed_step
    return
  fi

  if [ "$is_auto_mode" == true ]; then
    sudo rpm -ivh kmod-microsoft-hyper-v*.x86_64.rpm
  fi

  while true; do
    f_interactive_prompt "Please install kmod-microsoft-hyper-v by executing 'sudo rpm -ivh kmod-microsoft-hyper-v*.x86_64.rpm' and enter 'done' when done." "done"
    if [ "$?" -eq 0 ]; then
      f_increment_completed_step
      return
    fi
  done
}

#------------------------------------------------------------------------------

f_uninstall_kmod_hyperv_rpm() {
  # Find out if '$centos_release_str' represents centos 6.3 or lower.

  centos_release_str_arr=( $centos_release_str )
  centos_release_ver=${centos_release_str_arr[2]}

  # We are ok with string comparison. If centos release version is < 6.3, we
  # need to uninstall microsoft hyperv rpm.
  if [[ "$centos_release_ver" > "6.3" ]]; then
    f_decrement_completed_step
    return
  fi

  # In auto mode uninstall the kmod-microsoft-hyper-v-rhel6-4.1.x86_64 package without user prompt.
  if [ "$is_auto_mode" == true ]; then
    sudo rpm -e kmod-microsoft-hyper-v
  fi

  while true; do
    f_interactive_prompt "Please uninstall kmod-microsoft-hyper-v by execute 'sudo rpm -e kmod-microsoft-hyper-v' and enter 'done' when done." "done"
    if [ "$?" -eq 0 ]; then
      f_decrement_completed_step
      return
    fi
  done
}

#------------------------------------------------------------------------------
# Last Step : Persist Cohesity cloud migration flag
#------------------------------------------------------------------------------

# File name to presist cloud migration enabled flag.
cohesity_cloud_migration_flag_file="/etc/cohesity_cloud_migration_enabled"

f_install_cohesity_cloud_migration_flag() {
  touch $cohesity_cloud_migration_flag_file
  f_increment_completed_step
}

#------------------------------------------------------------------------------

f_uninstall_cohesity_cloud_migration_flag() {
  rm $cohesity_cloud_migration_flag_file
  f_decrement_completed_step
}

#------------------------------------------------------------------------------

f_parse_command_line_params_and_execute() {
  local auto_mode=$1
  local install_mode=$2
  local uninstall_mode=$3

  # If command line parameters are specified, we run the script in the auto mode
  # where all the changes to be done by the script are done automatically.
  if [ $auto_mode == true ]; then
    is_auto_mode=true
  fi

  if ([ $install_mode == true ] && [ $uninstall_mode = true ]); then
    echo "You can't install and uninstall at the same time."
    exit 1
  fi

  if [ $install_mode == true ]; then
    f_execute_install $completed_step
  elif [ $uninstall_mode == true ]; then
    f_execute_uninstall $completed_step
  else
    echo "This script will make changes needed for cloning your linux VM to Azure."
    echo "Do you want to install/uninstall the required changes?"
    read op

    if [ "$op" == "install" ]; then
      echo "Install start at step: $completed_step"
      f_execute_install $completed_step
    elif [ "$op" == "uninstall" ]; then
      echo "Uninstall start at step: $completed_step"
      f_execute_uninstall $completed_step
    else
      echo "You can either install or uninstall. $op is not a valid operation"
      exit 1
    fi
  fi
}
