#!/bin/bash
#
# Copyright 2017 Cohesity Inc.
#
# Configures the Centos 7 VM to be cloned to Azure. The script runs with help
# from our customer. It should be run with sudo privileges.
#
# NOTES:
#   1. It is an interactive shell.
#   2. The user can use to either install the changes or uninstall the changes.
#   3. During install we keep backup of files we modify in its directory in
#      .bak version of the file.
#   4. RPM packages need to installed or uninstalled manually by the user. We
#      just print the command to do the install or uninstall.
#   5. RPM packages are stored in well known directory.
#   6. We keep track of step we are at by persisting the step number on
#      successful execution of the step in 'completed_step' file.
#   7. On restart the user is supposed to run the script manually again.
#   8. On successful installation we touch /etc/cohesity_cloud_migration_enabled
#      in order to deterministically fail "convert and deploy" in case a VM,
#      where this script has not been run, is deployed to Azure.
#   9. On uninstall we delete /etc/cohesity_cloud_migration_enabled.
#
# Output: Exits with 0 upon successful configuration.
#         Exits with non-zero error code upon failure.

# Variable to capture content of /etc/redhat-release
centos_release_str=""

current_dir=$(dirname `readlink -f $0`)
. $current_dir/common_centos.sh

#------------------------------------------------------------------------------
# Method to install changes needed to clone a Centos 7 VM to Azure.
#
# The following configuration steps are taken from the Azure documentation
# article at https://goo.gl/vSuIZ5
#------------------------------------------------------------------------------

f_execute_install() {
  local completed_step="$1"

  while true; do
    case $completed_step in
      0 )
        f_install_etc_sysconfig_network ;;
      1 )
        f_install_etc_sysconfig_network_script_eth0 ;;
      2 )
        f_modify_udev_rules_for_ethernet_interface ;;
      3 )
        f_chkconfig_network_on ;;
      4 )
        f_install_python_rpm ;;
      5 )
        f_install_walinux_agent_rpm ;;
      6 )
        f_modify_boot_line ;;
      7 )
        f_rebuild_grub_during_install ;;
      8 )
        f_add_hyperv_module_to_initramfs ;;
      9 )
        f_rebuild_initramfs "installing" ;;
      10 )
        f_enable_waagent ;;
      11 )
        f_install_cohesity_cloud_migration_flag ;;
      12 )
        echo "Install completed"
        return 0 ;;
    esac
  done
}

#------------------------------------------------------------------------------
# Method to uninstall changes that were done to clone linux VM to Azure.
#------------------------------------------------------------------------------

f_execute_uninstall() {
  local completed_step="$1"

  while true; do
    case $completed_step in
      0 )
        echo "Uninstall completed"
        return 0 ;;
      1 )
        f_uninstall_etc_sysconfig_network ;;
      2 )
        f_uninstall_etc_sysconfig_network_script_eth0 ;;
      3 )
        f_restore_udev_rules_for_ethernet_interface ;;
      4 )
        f_restore_chkconfig_network_on ;;
      5 )
        f_uninstall_python_rpm ;;
      6 )
        f_uninstall_walinux_agent_rpm ;;
      7 )
        f_rebuild_grub_during_uninstall ;;
      8 )
        f_restore_boot_line ;;
      9 )
        f_rebuild_initramfs "uninstalling" ;;
      10 )
        f_restore_initramfs ;;
      11 )
        f_disable_waagent ;;
      12 )
        f_uninstall_cohesity_cloud_migration_flag ;;
    esac
  done
}

#------------------------------------------------------------------------------

# This script works for centos 7.
centos_release_file="/etc/redhat-release"
expected_centos_release_prefix="CentOS Linux release 7"

if [ -f $centos_release_file ]; then
  centos_release_str=$(cat $centos_release_file)
fi

if [[ "$centos_release_str" != "$expected_centos_release_prefix"* ]]; then
  echo "This file supports '$expected_centos_release_prefix', found '$centos_release_str'"
  exit 1
fi

# parse command line parameters for auto and install/uninstall mode.
auto_mode=false
install_mode=false
uninstall_mode=false

while test $# -gt 0
do
    case "$1" in
        --auto) auto_mode=true
            ;;
        --install) install_mode=true
            ;;
        --uninstall) uninstall_mode=true
            ;;
        --*) echo "bad option $1. Exiting..."
              exit 1
            ;;
    esac
    shift
done

f_parse_command_line_params_and_execute $auto_mode $install_mode $uninstall_mode

# At the successful finish we will call sync to flush all the changes we have
# made to the filesystem.
sync
exit 0

# TBD(vipin):
#     Do not create swap space on the OS disk.
#     The Azure Linux Agent can automatically configure swap space using the
#     local resource disk that is attached to the VM after provisioning on
#     Azure. Note that the local resource disk is a temporary disk, and might
#     be emptied when the VM is deprovisioned. After installing the Azure Linux
#     Agent (see previous step), modify the following parameters in
#     /etc/waagent.conf appropriately:
#
# We don't want swap, so we'll use the defaults (shown below). This is a no-op.
#   ResourceDisk.Format=y
#   ResourceDisk.Filesystem=ext4
#   ResourceDisk.MountPoint=/mnt/resource
#   ResourceDisk.EnableSwap=n
#   ResourceDisk.SwapSizeMB=0
