#!/bin/bash
#
# Copyright 2018 Cohesity Inc.
#
# Author: Shishir Jindal(sjindal@cohesity.com)
#
# Configures the RHEL 7 VM to be cloned to Azure. The script runs with help
# from our customer. It should be run with sudo privileges.
#
# NOTES:
#   1. It is an interactive shell.
#   2. The user can use to either install the changes or uninstall the changes.
#   3. During install we keep backup of files we modify in its directory in
#      .bak version of the file.
#   4. RPM packages need to installed or uninstalled manually by the user. We
#      just print the command to do the install or uninstall.
#   5. RPM packages are stored in well known directory.
#   6. We keep track of step we are at by persisting the step number on
#      successful execution of the step in 'completed_step' file.
#   7. On restart the user is supposed to run the script manually again.
#
# Output: Exits with 0 upon successful configuration.
#         Exits with non-zero error code upon failure.

# Variable to capture content of /etc/redhat-release.
rhel_release_str=""
version=7

current_dir=$(dirname `readlink -f $0`)
. $current_dir/common_rhel.sh

#------------------------------------------------------------------------------
# Method to install changes needed to clone a RHEL 7 VM to Azure.
#
# The following configuration steps are taken from the Azure documentation
# article at https://docs.microsoft.com/en-us/azure/virtual-machines/linux/
# redhat-create-upload-vhd.
#------------------------------------------------------------------------------

f_execute_install() {
  local completed_step="$1"

  while true; do
    case $completed_step in
      0 )
        f_install_etc_sysconfig_network ;;
      1 )
        f_install_etc_sysconfig_network_script_eth0_v7 ;;
      2 )
        f_enable_network_service ;;
      3 )
        f_check_redhat_subscription ;;
      4 )
        f_modify_boot_line_for_rhel7 ;;
      5 )
        f_rebuild_grub_during_install ;;
      6 )
        f_modify_dracut_config_file_v7 ;;
      7 )
        f_rebuild_initramfs "installing" ;;
      8 )
        f_enable_ssh_server ;;
      9 )
        f_modify_sshd_config_file ;;
      10 )
        f_enable_redhat_extras_repository "$version" ;;
      11 )
        f_install_azure_linux_agent_v7 ;;
      12 )
        f_modify_waagent_conf_file ;;
      13 )
        f_install_cohesity_cloud_migration_flag ;;
      14 )
        echo "Install completed"
        return 0 ;;
    esac
  done
}

#------------------------------------------------------------------------------
# Method to uninstall changes that were done to clone linux VM to Azure.
#------------------------------------------------------------------------------

f_execute_uninstall() {
  local completed_step="$1"

  while true; do
    case $completed_step in
      0 )
        echo "Uninstall completed"
        return 0 ;;
      1 )
        f_uninstall_etc_sysconfig_network ;;
      2 )
        f_uninstall_etc_sysconfig_network_script_eth0 ;;
      3 )
        f_disable_network_service ;;
      4 )
        f_restore_boot_line_for_rhel7 ;;
      5 )
        f_rebuild_grub_during_uninstall ;;
      6 )
        f_restore_dracut_config_file ;;
      7 )
        f_rebuild_initramfs "uninstalling" ;;
      8 )
        f_disable_ssh_server ;;
      9 )
        f_restore_sshd_config_file ;;
      10 )
        f_disable_redhat_extras_repository "$version" ;;
      11 )
        f_uninstall_azure_linux_agent ;;
      12 )
        f_restore_waagent_conf_file ;;
      13 )
        f_disable_waagent_v7 ;;
      14 )
        f_uninstall_cohesity_cloud_migration_flag ;;
    esac
  done
}

#------------------------------------------------------------------------------

# This script works for rhel 7.
rhel_release_file="/etc/redhat-release"
expected_rhel_release_prefix="Red Hat Enterprise Linux Server release 7"

if [ -f $rhel_release_file ]; then
  rhel_release_str=$(cat $rhel_release_file)
fi

if [[ "$rhel_release_str" != "$expected_rhel_release_prefix"* ]]; then
  echo "This file supports '$expected_rhel_release_prefix', found '$rhel_release_str'"
  exit 1
fi

# parse command line parameters for auto and install/uninstall mode.
auto_mode=false
install_mode=false
uninstall_mode=false

while test $# -gt 0
do
    case "$1" in
        --auto) auto_mode=true
            ;;
        --install) install_mode=true
            ;;
        --uninstall) uninstall_mode=true
            ;;
        --*) echo "bad option $1. Exiting..."
              exit 1
            ;;
    esac
    shift
done

f_parse_command_line_params_and_execute $auto_mode $install_mode $uninstall_mode

# At the successful finish we will call sync to flush all the changes we have
# made to the filesystem.
sync
exit 0
