#!/bin/bash
#
# Copyright 2017 Cohesity Inc.
#
# Common code shared by scripts configuring various flavors of Linux.

current_dir=$(dirname `readlink -f $0`)
. $current_dir/common.sh

#------------------------------------------------------------------------------

# Function to install python2-pyasn1.
f_install_python_rpm() {
  # In auto mode install the python2-pyasn1 package without user prompt. RPM
  # is expected to be present in the current directory.
  if [ "$is_auto_mode" == true ]; then
    sudo rpm -ivh python2-pyasn1*.el7.noarch.rpm
    f_increment_completed_step
    return
  fi

  # In non auto mode user should execute the commands manually.
  while true; do
    echo "Please install python2-pyasn1 by executing 'sudo rpm -ivh python2-pyasn1*.el7.noarch.rpm' and enter 'done' when done."
    read op
    if [ "$op" == "done" ]; then
      f_increment_completed_step
      return
    fi
  done
}

#------------------------------------------------------------------------------

# Function to uninstall python2-pyasn1.
f_uninstall_python_rpm() {
  # There is no auto mode to uninstall. The user needs to manually uninstall
  # it.
  while true; do
    echo "Please uninstall python2-pyasn1 by execute 'sudo rpm -e python2-pyasn1' and enter 'done' when done."
    read op
    if [ "$op" == "done" ]; then
      f_decrement_completed_step
      return
    fi
  done
}

#------------------------------------------------------------------------------

# Function to install WALinuxAgent.
f_install_walinux_agent_rpm() {
  # In auto mode install the WALinuxAgent package without user prompt. RPM
  # is expected to be present in the current directory.
  if [ "$is_auto_mode" == true ]; then
    sudo rpm -ivh WALinuxAgent*.el7.centos.noarch.rpm
    f_increment_completed_step
    return
  fi

  while true; do
    echo "Please install WALinuxAgent by executing 'sudo rpm -ivh WALinuxAgent-2.2.0-1.el7.centos.noarch.rpm' and enter 'done' when done."
    read op
    if [ "$op" == "done" ]; then
      f_increment_completed_step
      return
    fi
  done
}

#------------------------------------------------------------------------------

# Function to uninstall WALinuxAgent.
f_uninstall_walinux_agent_rpm() {
  # In auto mode uninstall the WALinuxAgent package without user prompt.
  if [ "$is_auto_mode" == true ]; then
    sudo rpm -e WALinuxAgent
    f_decrement_completed_step
    return
  fi

  while true; do
    echo "Please uninstall WALinuxAgent by execute 'sudo rpm -e WALinuxAgent' and enter 'done' when done."
    read op
    if [ "$op" == "done" ]; then
      f_decrement_completed_step
      return
    fi
  done
}

#------------------------------------------------------------------------------

# Function to overwrite /etc/default/grub file.
#
# Modify the kernel boot line in your grub configuration to include
# additional kernel parameters for Azure. This will also ensure all console
# messages are sent to the first serial port, which can assist Azure
# support with debugging issues. It also turns off the new CentOS 7 naming
# conventions for NICs.
#
# Graphical and quiet boot are not useful in a cloud environment where we
# want all the logs to be sent to the serial port.
f_overwrite_etc_default_grub() {
  replacementline='GRUB_CMDLINE_LINUX="crashkernel=auto fips=$fips boot=UUID=$bootUuid rootdelay=300 console=ttyS0 earlyprintk=ttyS0 net.ifnames=0 elevator=noop"'
  f_match_replace "GRUB_CMDLINE_LINUX=" /etc/default/grub "${replacementline}"
}

#------------------------------------------------------------------------------

# Function to modify boot line in grub config.
#
# replacementline='GRUB_CMDLINE_LINUX="crashkernel=auto rootdelay=300 '\
# 'console=ttyS0 earlyprintk=ttyS0 net.ifnames=0 elevator=noop"'
# match_replace "GRUB_CMDLINE_LINUX=" /etc/default/grub "${replacementline}"
f_modify_boot_line() {
  f_interactive_prompt "We will overwrite 'GRUB_CMDLINE_LINUX=...' line in /etc/default/grub. Are you sure? (y/n)" "y"
  if [ "$?" -eq 0 ]; then
    # /etc/default/grub should be found and the backup should succeed
    f_backup /etc/default/grub
    f_overwrite_etc_default_grub
    f_increment_completed_step
  else
    echo "exiting"
    exit 0
  fi
}

#------------------------------------------------------------------------------

# Function to restore boot line in grub config.
f_restore_boot_line() {
  if [ ! -f /etc/default/grub.bak ]; then
    echo "/etc/default/grub.bak not found. This is an error. Exiting."
    exit 1
  fi

  f_recover "/etc/default/grub"
  f_decrement_completed_step
}

#------------------------------------------------------------------------------

# Function to overwrite /etc/dracut.conf file.
f_overwrite_etc_dracut_conf() {
sudo tee /etc/dracut.conf > /dev/null <<'EOF'

add_drivers+=" hv_vmbus hv_netvsc hv_storvsc "
EOF
}

#------------------------------------------------------------------------------

# Function to add hyperv module to initramfs.
f_add_hyperv_module_to_initramfs() {
  f_interactive_prompt "We will add hyperv module to initramfs by adding a line to /etc/dracut.conf. Are you sure? (y/n)" "y"
  if [ "$?" -eq 0 ]; then
    # /etc/dracut.conf should exist and the backup of that file should succeed.
    f_backup "/etc/dracut.conf"
    f_overwrite_etc_dracut_conf
    f_increment_completed_step
  else
    echo "exiting"
    exit 0
  fi
}

#------------------------------------------------------------------------------

# Function to restore /etc/dracut.conf.
f_restore_initramfs() {
  if [ ! -f /etc/dracut.conf.bak ]; then
    echo "/etc/dracut.conf.bak not found. This is an error. Exiting."
    exit 1
  fi

  f_recover "/etc/dracut.conf"
  f_decrement_completed_step
}

#------------------------------------------------------------------------------

# Function to add/remove hyperv module to initramfs.
#
# If the first argument ($1) is 'installing' it increments the completed step
# after reconfiguring initramfs by running dracut, else it decrements the
# completed step after running dracut.
f_rebuild_initramfs() {
  local operation="$1"

  # In auto mode we will rebuild grub automatically.
  if [ "$is_auto_mode" == true ]; then
    sudo dracut -f -v --no-hostonly
    if [ "$operation" == "installing" ]; then
      f_increment_completed_step
    else
      f_decrement_completed_step
    fi
    return
  fi

  while true; do
    # Rebuild initramfs.
    # Disable Host-Only mode to generate generic images. Redhat-7 default mode
    # is hostonly.
    # dracut -f -v --no-hostonly
    echo "Please execute 'sudo dracut -f -v --no-hostonly' and enter 'done' when done."
    read op
    if [ "$op" == "done" ]; then
      if [ "$operation" == "installing" ]; then
        f_increment_completed_step
      else
        f_decrement_completed_step
      fi
      return
    fi
  done
}

#------------------------------------------------------------------------------

# Function to enable waagent.
f_enable_waagent() {
  # In auto mode we will enable waagent automatically.
  if [ "$is_auto_mode" == true ]; then
    sudo systemctl enable waagent
    f_increment_completed_step
    return
  fi

  while true; do
    echo "Please execute: 'sudo systemctl enable waagent' and enter 'done' when done."

    # We will not ask waagnet to be deporvisioed since it causes problems with
    # sshd. The following line had been commented as a result.
    # echo "'sudo waagent -force -deprovision' and enter 'done' when done."
    read op
    if [ "$op" == "done" ]; then
      f_increment_completed_step
      return
    fi
  done
}

#------------------------------------------------------------------------------

# Function to disable waagent.
f_disable_waagent() {
  # In auto mode we will enable waagent automatically.
  if [ "$is_auto_mode" == true ]; then
    sudo systemctl disable waagent
    f_decrement_completed_step
    return
  fi

  while true; do
    echo "Please execute 'sudo systemctl disable waagent'and enter 'done' when done."
    read op
    if [ "$op" == "done" ]; then
      f_decrement_completed_step
      return
    fi
  done
}

#------------------------------------------------------------------------------

# Function to prompt, confirm and execute
#   ln -snf /dev/null /etc/udev/rules.d/75-persistent-net-generator.rules
#   rm -f /etc/udev/rules.d/70-persistent-net.rules
#
# Modify udev rules to avoid generating static rules for the Ethernet
# interface(s). These rules can cause problems when cloning a virtual
# machine in Microsoft Azure or Hyper-V:
f_modify_udev_rules_for_ethernet_interface_for_centos6() {
  f_interactive_prompt "We will overwrite /etc/udev/rules.d/75-persistent-net-generator.rules.\nAre you sure? (y/n)" "y"
  if [ "$?" -eq 0 ]; then
    # Backup will fail if the file is not found and we are ok with that.
    f_backup "/etc/udev/rules.d/75-persistent-net-generator.rules" || true

    # It is ok to run the following command multiple times in case of error.
    sudo ln -snf /dev/null /etc/udev/rules.d/75-persistent-net-generator.rules

    # Backup will fail if the file is not found and we are ok with that.
    f_backup "/etc/udev/rules.d/70-persistent-net.rules" || true

    # The following command will fail if the file is not found and we are ok
    # with that.
    sudo rm -f /etc/udev/rules.d/70-persistent-net.rules || true

    f_increment_completed_step
  else
    echo "exiting"
    exit 0
  fi
}

#------------------------------------------------------------------------------

# Function to undo the work done by f_modify_udev_rules_for_ethernet_interface_for_centos6
f_restore_udev_rules_for_ethernet_interface_for_centos6() {
  f_recover "/etc/udev/rules.d/75-persistent-net-generator.rules"
  f_recover "/etc/udev/rules.d/70-persistent-net.rules"
  f_decrement_completed_step
}

#------------------------------------------------------------------------------

# Function to overwrite /boot/grub/menu.lst file.
#
# Modify the kernel boot line in your grub configuration to include additional
# kernel parameters for Azure. To do this, open /boot/grub/menu.lst in a text
# editor and ensure that the default kernel includes the following parameters:
#   console=ttyS0 earlyprintk=ttyS0 rootdelay=300
#
# This will also ensure all console messages are sent to the first serial port,
# which can assist Azure support with debugging issues.
f_overwrite_boot_grub_menu() {
  default_kernel_params="console=ttyS0 earlyprintk=ttyS0 rootdelay=300"

  # Find out if '$centos_release_str' represents centos 6.5 or lower.

  centos_release_str_arr=( $centos_release_str )
  centos_release_ver=${centos_release_str_arr[2]}

  # We are ok with string comparison. If centos release version is <= 6.5, we
  # need to add numa=off.
  if [[ ! "$centos_release_ver" > "6.5" ]]; then
    default_kernel_params="console=ttyS0 earlyprintk=ttyS0 rootdelay=300 numa=off"
  fi

  # TODO(vipin): Automate the following

  while true; do
    echo "Please add '"$default_kernel_params"' as parameters for default kernel in /boot/grub/menu.lst and enter 'done' when done."
    read op
    if [ "$op" == "done" ]; then
      return
    fi
  done
}

#------------------------------------------------------------------------------

# Function to modify boot line in grub config for centos 6.
f_modify_boot_line_for_centos6() {
  f_interactive_prompt "We will overwrite default kernel parameters in /boot/grub/menu.lst. Are you sure? (y/n)" "y"
  if [ "$?" -eq 0 ]; then
    # /boot/grub/menu.lst should be found and the backup should succeed
    f_backup /boot/grub/menu.lst
    f_overwrite_boot_grub_menu
    f_increment_completed_step
  else
    echo "exiting"
    exit 0
  fi
}

#------------------------------------------------------------------------------

# Function to restore /boot/grub/menu.lst.
f_restore_boot_line_for_centos6() {
  if [ ! -f /boot/grub/menu.lst.bak ]; then
    echo "/boot/grub/menu.lst.bak not found. This is an error. Exiting."
    exit 1
  fi

  f_recover "/boot/grub/menu.lst"
  f_decrement_completed_step
}

#------------------------------------------------------------------------------

# Function to install hyperv for centos 6.3 or earlier.
f_install_hyperv_rpm() {
  # Find out if '$centos_release_str' represents centos 6.3 or lower.

  centos_release_str_arr=( $centos_release_str )
  centos_release_ver=${centos_release_str_arr[2]}

  # We are ok with string comparison. If centos release version is <= 6.3, we
  # need to install microsoft hyperv rpm.
  if [[ "$centos_release_ver" > "6.3" ]]; then
    f_increment_completed_step
    return
  fi

  if [ "$is_auto_mode" == true ]; then
    sudo rpm -ivh microsoft-hyper-v-*.x86_64.rpm
    f_increment_completed_step
    return
  fi

  while true; do
    echo "Please install microsoft-hyperv by executing 'sudo rpm -ivh microsoft-hyper-v-*.x86_64.rpm' and enter 'done' when done."
    read op
    if [ "$op" == "done" ]; then
      f_increment_completed_step
      return
    fi
  done
}

#------------------------------------------------------------------------------

# Function to uninstall hyperv for centos 6.3 or earlier.
f_uninstall_hyperv_rpm() {
  # Find out if '$centos_release_str' represents centos 6.3 or lower.

  centos_release_str_arr=( $centos_release_str )
  centos_release_ver=${centos_release_str_arr[2]}

  # We are ok with string comparison. If centos release version is <= 6.3, we
  # need to uninstall microsoft hyperv rpm.
  if [[ "$centos_release_ver" > "6.3" ]]; then
    f_decrement_completed_step
    return
  fi

  # In auto mode uninstall the microsoft-hyper-v-rhel6-4.1.x86_64 package without user prompt.
  if [ "$is_auto_mode" == true ]; then
    sudo rpm -e microsoft-hyper-v
    f_decrement_completed_step
    return
  fi

  while true; do
    echo "Please uninstall microsoft-hyperv by executing 'sudo rpm -e microsoft-hyper-v' and enter 'done' when done."
    read op
    if [ "$op" == "done" ]; then
      f_decrement_completed_step
      return
    fi
  done
}

#------------------------------------------------------------------------------

# Function to install hyperv for centos 6.3 or earlier.
f_install_kmod_hyperv_rpm() {
  # Find out if '$centos_release_str' represents centos 6.3 or lower.

  centos_release_str_arr=( $centos_release_str )
  centos_release_ver=${centos_release_str_arr[2]}

  # We are ok with string comparison. If centos release version is < 6.3, we
  # need to install microsoft hyperv rpm.
  if [[ "$centos_release_ver" > "6.3" ]]; then
    f_increment_completed_step
    return
  fi

  if [ "$is_auto_mode" == true ]; then
    sudo rpm -ivh kmod-microsoft-hyper-v*.x86_64.rpm
    f_increment_completed_step
    return
  fi

  while true; do
    echo "Please install kmod-microsoft-hyper-v by executing 'sudo rpm -ivh kmod-microsoft-hyper-v*.x86_64.rpm' and enter 'done' when done."
    read op
    if [ "$op" == "done" ]; then
      f_increment_completed_step
      return
    fi
  done
}

#------------------------------------------------------------------------------

# Function to uninstall hyperv for centos 6.3 or earlier.
f_uninstall_kmod_hyperv_rpm() {
  # Find out if '$centos_release_str' represents centos 6.3 or lower.

  centos_release_str_arr=( $centos_release_str )
  centos_release_ver=${centos_release_str_arr[2]}

  # We are ok with string comparison. If centos release version is < 6.3, we
  # need to uninstall microsoft hyperv rpm.
  if [[ "$centos_release_ver" > "6.3" ]]; then
    f_decrement_completed_step
    return
  fi

  # In auto mode uninstall the kmod-microsoft-hyper-v-rhel6-4.1.x86_64 package without user prompt.
  if [ "$is_auto_mode" == true ]; then
    sudo rpm -e kmod-microsoft-hyper-v
    f_decrement_completed_step
    return
  fi

  while true; do
    echo "Please uninstall kmod-microsoft-hyper-v by execute 'sudo rpm -e kmod-microsoft-hyper-v' and enter 'done' when done."
    read op
    if [ "$op" == "done" ]; then
      f_decrement_completed_step
      return
    fi
  done
}

#------------------------------------------------------------------------------
