#!/bin/bash
#
# Copyright 2018 Cohesity Inc.
#
# Author : Shishir Jindal(sjindal@cohesity.com)
# Common code shared by scripts configuring various flavors of Redhat Linux.

current_dir=$(dirname `readlink -f $0`)
. $current_dir/common.sh

#------------------------------------------------------------------------------

# Function to check for redhat subscription status.
f_check_redhat_subscription() {
  local status=`subscription-manager status`
  if [[ $status != *"Current"* ]]; then
    echo "This system is not registered with redhat subscription."
    exit 1
  fi
  f_increment_completed_step
}

#------------------------------------------------------------------------------

# Function to enable red hat extras repository.
f_enable_redhat_extras_repository() {
  local version="$1"

  # In auto mode just enable the red hat extras repository.
  if [ "$is_auto_mode" == true ]; then
    sudo subscription-manager repos --enable=rhel-$version-server-extras-rpms
    f_increment_completed_step
    return
  fi

  # In non auto mode user should enter the registering command.
  while true; do
    echo "Please enable Red Hat extras repository by executing 'sudo subscription-manager repos --enable=rhel-"$version"-server-extras-rpms' and enter 'done' when done."
    read op
    if [ "$op" == "done" ]; then
      f_increment_completed_step
      return
    fi
  done
}

#------------------------------------------------------------------------------

# Function to disable red hat extras repository.
f_disable_redhat_extras_repository() {
  local version="$1"

  # In auto mode just disable the red hat extras repository.
  if [ "$is_auto_mode" == true ]; then
    subscription-manager repos --disable=rhel-$version-server-extras-rpms
    f_decrement_completed_step
    return
  fi

  # In non auto mode user should enter the registering command.
  while true; do
    echo "Please disable Red Hat extras repository by executing 'subscription-manager repos --disable=rhel-"$version"-server-extras-rpms' and enter 'done' when done."
    if [ "$op" == "done" ]; then
      f_decrement_completed_step
      return
    fi
  done
}

#-----------------------------------------------------------------------------

# Function to overwrite /etc/default/grub file automatically for RHEL6.
# Modify the kernel boot line in your grub configuration to include additional
# kernel parameters for Azure. To do this, open /etc/default/grub in a text
# editor and ensure that the default kernel includes the following parameters:
#   rootdelay=300 console=ttyS0 earlyprintk=ttyS0
#
# This will also ensure all console messages are sent to the first serial port,
# which can assist Azure support with debugging issues.
f_overwrite_boot_default_group_automatically() {
  local default_kernel_params="$1"
  f_match_replace "${default_kernel_params}" /etc/default/grub "${default_kernel_params}"
}

# Function to overwrite /etc/default/grub file.
f_overwrite_boot_default_group() {
  default_kernel_params="rootdelay=300 console=ttyS0 earlyprintk=ttyS0"

  # In auto mode just modify boot line automatically.
  if [ "$is_auto_mode" == true ]; then
    f_overwrite_boot_default_group_automatically "$default_kernel_params"
    return
  fi

  # In non auto mode user should manually modify the boot line.
  while true; do
    echo "Please add '"$default_kernel_params"' as parameters for default kernel in /etc/default/grub and enter 'done' when done."
    read op
    if [ "$op" == "done" ]; then
      return
    fi
  done
}

#------------------------------------------------------------------------------

# Function to modify boot line in grub config for RHEL 6.
f_modify_boot_line_for_rhel6() {
  f_interactive_prompt "We will overwrite default kernel parameters in /etc/default/grub. Are you sure? (y/n)" "y"
  if [ "$?" -eq 0 ]; then
    # /etc/default/grub should be found and the backup should succeed.
    f_backup /etc/default/grub
    f_overwrite_boot_default_group
    f_increment_completed_step
  else
    echo "exiting"
    exit 0
  fi
}

#------------------------------------------------------------------------------

# Function to restore /etc/default/grub.
f_restore_boot_line_for_rhel6() {
  if [ ! -f /etc/default/grub.bak ]; then
    echo "/etc/default/grub.bak not found. This is an error. Exiting."
    exit 1
  fi

  f_recover "/etc/default/grub"
  f_decrement_completed_step
}

#------------------------------------------------------------------------------

# Function to ensure that ssh server is enabled at boot time.
f_enable_ssh_server() {
  # User should enable the ssh server manually.
  while true; do
    echo "Please enable ssh server at boot time by executing 'sudo chkconfig sshd on' and enter 'done' when done."
    read op
    if [ "$op" == "done" ]; then
      f_increment_completed_step
      return
    fi
  done
}

#------------------------------------------------------------------------------

# Function to disable ssh server.
f_disable_ssh_server() {
  # User should disable the ssh server manually.
  while true; do
    echo "Please disable ssh server by executing 'sudo chkconfig sshd off' and enter 'done' when done."
    echo "Disabling sshd will disable the remote accesss to the VM."
    read op
    if [ "$op" == "done" ]; then
      f_decrement_completed_step
      return
    fi
  done
}

#------------------------------------------------------------------------------

# Function to overwrite /etc/ssh/sshd_config file.
f_overwrite_sshd_config_file() {
  local replacementline="ClientAliveInterval 180"
  f_match_replace "ClientAliveInterval" /etc/ssh/sshd_config "${replacementline}"
}

#------------------------------------------------------------------------------

# Function to modify sshd_config file.
f_modify_sshd_config_file() {
  # In auto mode just overwrite the file automatically.
  if [ "$is_auto_mode" == true  ]; then
    f_interactive_prompt "We will overwrite /etc/ssh/sshd_config. Are you sure? (y/n)" "y"
    if [ "$?" -eq 0 ]; then
      # /etc/ssh/sshd_config should be found and the backup should succeed.
      f_backup /etc/ssh/sshd_config
      f_overwrite_sshd_config_file
      f_increment_completed_step
      return
    else
      echo "exiting"
      exit 0
    fi
  fi

  # In non auto mode user should manually edit the sshd_config file.
  while true; do
    echo "Please modify /etc/ssh/sshd_config to include the following line 'ClientAliveInterval 180' and enter 'done' when done."
    read op
    if [ "$op" == "done" ]; then
      f_increment_completed_step
      return
    fi
  done
}

#------------------------------------------------------------------------------

# Function to restore /etc/ssh/sshd_config.
f_restore_sshd_config_file() {
  if [ ! -f /etc/ssh/sshd_config ]; then
    echo "/etc/ssh/sshd_config.bak not found. This is an error. Exiting."
    exit 1
  fi

  f_recover "/etc/ssh/sshd_config"
  f_decrement_completed_step
}

#------------------------------------------------------------------------------

# Function to install Azure linux Agent for rhel6.
f_install_azure_linux_agent_v6() {
  # In auto mode just install azure linux agent automatically.
  if [ "$is_auto_mode" == true ]; then
    sudo yum install WALinuxAgent
    sudo chkconfig waagent on
    f_increment_completed_step
    return
  fi

  # In non auto mode user should enter the commands manually to install azure
  # linux agent.
  while true; do
    echo "Please install Azure Linux Agent by executing 'sudo yum install WALinuxAgent'"
    echo "Please execute 'sudo chkconfig waagent on' and enter 'done' when done."
    read op
    if [ "$op" == "done" ]; then
      f_increment_completed_step
      return
    fi
  done
}

#------------------------------------------------------------------------------

# Function to install Azure linux Agent for rhel7.
f_install_azure_linux_agent_v7() {
  # In auto mode just install azure linux agent automatically.
  if [ "$is_auto_mode" == true ]; then
    sudo yum install WALinuxAgent
    sudo systemctl enable waagent.service
    f_increment_completed_step
    return
  fi

  # In non auto mode user should enter the commands manually to install azure
  # linux agent.
  while true; do
    echo "Please install Azure Linux Agent by executing 'sudo yum install WALinuxAgent'"
    echo "Please execute 'sudo systemctl enable waagent.service' and enter 'done' when done."
    read op
    if [ "$op" == "done" ]; then
      f_increment_completed_step
      return
    fi
  done
}

#------------------------------------------------------------------------------

# Function to uninstall Azure linux Agent for rhel7.
f_uninstall_azure_linux_agent() {
  # In auto mode uninstall the WALinuxAgent package without user prompt.
  if [ "$is_auto_mode" == true ]; then
    sudo rpm -e WALinuxAgent
    f_decrement_completed_step
    return
  fi

  while true; do
    echo "Please uninstall azure linux agent (WALinuxAgent) by executing
    'sudo rpm -e WALinuxAgent' and enter 'done' when done."
    read op
    if [ "$op" == "done" ]; then
      f_decrement_completed_step
      return
    fi
  done
}

#------------------------------------------------------------------------------

# Function to append parameters in /etc/waagent.conf file.
#
# The Azure Linux Agent can automatically configure swap space using the
# local resource disk that is attached to the VM after provisioning on
# Azure. Note that the local resource disk is a temporary disk, and might
# be emptied when the VM is deprovisioned. After installing the Azure Linux
# Agent (see previous step), modify the following parameters in
# /etc/waagent.conf appropriately:
f_append_waagent_conf_file() {
  local replacementline="ResourceDisk.Format=y"
  f_match_replace "ResourceDisk.Format" /etc/waagent.conf "${replacementline}"

  replacementline="ResourceDisk.Filesystem=ext4"
  f_match_replace "ResourceDisk.Filesystem" /etc/waagent.conf "${replacementline}"

  replacementline="ResourceDisk.MountPoint=\/mnt\/resource"
  f_match_replace "ResourceDisk.MountPoint" /etc/waagent.conf "${replacementline}"

  replacementline="ResourceDisk.EnableSwap=y"
  f_match_replace "ResourceDisk.EnableSwap" /etc/waagent.conf "${replacementline}"

  replacementline="ResourceDisk.SwapSizeMB=2048"
  f_match_replace "ResourceDisk.SwapSizeMB" /etc/waagent.conf "${replacementline}"
}

#------------------------------------------------------------------------------

# Function to modify parameters in waagent configuration file.
f_modify_waagent_conf_file() {
  f_interactive_prompt "We will append /etc/waagent.conf. Are you sure? (y/n)" "y"
  if [ "$?" -eq 0 ]; then
    # /etc/ssh/sshd_config should be found and the backup should succeed.
    f_backup /etc/waagent.conf
    f_append_waagent_conf_file
    f_increment_completed_step
  else
    echo "exiting"
    exit 0
  fi
}

#------------------------------------------------------------------------------

# Function to restore /etc/waagent.conf.
f_restore_waagent_conf_file() {
  if [ ! -f /etc/waagent.conf ]; then
    echo "/etc/waagent.conf.bak not found. This is an error. Exiting."
    exit 1
  fi

  f_recover "/etc/waagent.conf"
  f_decrement_completed_step
}

#------------------------------------------------------------------------------

# Function to de-provision the virtual machine.
f_deprovision_vm() {
  # In auto mode just de-provision the virtual machine.
  if [ "$is_auto_mode" == true ]; then
    sudo waagent -force -deprovision
    f_increment_completed_step
    return
  fi

  # In non auto mode user should manually de-provision the virtual machine.
  while true; do
    echo "Please deprovision the virtual machine by executing 'sudo waagent -force -deprovision' and enter 'done' when done."
    read op
    if [ "$op" == "done" ]; then
      f_increment_completed_step
      return
    fi
  done
}

#------------------------------------------------------------------------------

# Function to disable waagent for rhel6.
f_disable_waagent_v6() {
  # In auto mode just disable waagent automatically.
  if [ "$is_auto_mode" == true ]; then
    sudo chkconfig waagent off
    f_decrement_completed_step
    return
  fi

  # In non auto mode user has to execute the commands manually.
  while true; do
    echo "Please disable waagent by executing 'sudo chkconfig waagent off' and enter 'done' when done."
    read op
    if [ "$op" == "done" ]; then
      f_decrement_completed_step
      return
    fi
  done
}

#------------------------------------------------------------------------------

# Function to disable waagent for rhel7.
f_disable_waagent_v7() {
  # In auto mode just disable waagent automatically.
  if [ "$is_auto_mode" == true ]; then
    sudo systemctl disable waagent
    f_decrement_completed_step
    return
  fi

  # In non auto mode user has to execute the commands manually.
  while true; do
    echo "Please disable waagent by executing 'sudo systemctl disable waagent' and enter 'done' when done."
    read op
    if [ "$op" == "done" ]; then
      f_decrement_completed_step
      return
    fi
  done
}

#------------------------------------------------------------------------------

# Function to enable network.
f_enable_network_service() {
  # In auto mode we will enable network automatically.
  if [ "$is_auto_mode" == true ]; then
    sudo systemctl enable network
    f_increment_completed_step
    return
  fi

  # In non auto mode user has to execute the commands manually.
  while true; do
    echo "Please execute 'sudo systemctl enable network'and enter 'done' when done."
    read op
    if [ "$op" == "done" ]; then
      f_increment_completed_step
      return
    fi
  done
}

#------------------------------------------------------------------------------

# Function to disable network.
f_disable_network_service() {
  # User should manually execute the commands to diable the network.
  while true; do
    echo "Please execute 'sudo systemctl disable network'and enter 'done' when done."
    echo "Disabling the network service will stop remote access to the VM."
    read op
    if [ "$op" == "done" ]; then
      f_decrement_completed_step
      return
    fi
  done
}

#------------------------------------------------------------------------------

# Function to overwrite /etc/default/grub file.
#
# Modify the kernel boot line in your grub configuration to include additional
# kernel parameters for Azure. To do this modification, open /etc/default/grub
# in a text editor, and edit the GRUB_CMDLINE_LINUX parameter.
f_overwrite_etc_default_grub_for_rhel7() {
  replacementline='GRUB_CMDLINE_LINUX="rootdelay=300 console=ttyS0 earlyprintk=ttyS0 net.ifnames=0"'
  f_match_replace "GRUB_CMDLINE_LINUX=" /etc/default/grub "${replacementline}"
}

#------------------------------------------------------------------------------

# Function to modify boot line in grub config.
f_modify_boot_line_for_rhel7() {
  f_interactive_prompt "We will overwrite 'GRUB_CMDLINE_LINUX=...' line in /etc/default/grub. Are you sure? (y/n)" "y"
  if [ "$?" -eq 0 ]; then
    # /etc/default/grub should be found and the backup should succeed.
    f_backup /etc/default/grub
    f_overwrite_etc_default_grub_for_rhel7
    f_increment_completed_step
  else
    echo "exiting"
    exit 0
  fi
}

#------------------------------------------------------------------------------

# Function to restore boot line in grub config.
f_restore_boot_line_for_rhel7() {
  if [ ! -f /etc/default/grub.bak ]; then
    echo "/etc/default/grub.bak not found. This is an error. Exiting."
    exit 1
  fi

  f_recover "/etc/default/grub"
  f_decrement_completed_step
}

#------------------------------------------------------------------------------

# Function to overwrite /etc/sysconfig/network-scripts/ifcfg-eth0.
f_overwrite_etc_sysconfig_network_script_eth0_v7() {
# The following code has to start at the beginning of the line.
sudo tee /etc/sysconfig/network-scripts/ifcfg-eth0 > /dev/null <<'EOF'
DEVICE=eth0
ONBOOT=yes
BOOTPROTO=dhcp
PERSISTENT_DHCLIENT=yes
TYPE=Ethernet
USERCTL=no
PEERDNS=yes
IPV6INIT=no
NM_CONTROLLED=no
EOF
}

#------------------------------------------------------------------------------

# Function to prompt and confirm while modifying /etc/sysconfig/network-scripts/ifcfg-eth0.
f_install_etc_sysconfig_network_script_eth0_v7() {
  f_interactive_prompt "We will overwrite /etc/sysconfig/network-scripts/ifcfg-eth0. \nAre you sure? (y/n)" "y"
  if [ "$?" -eq 0 ]; then
    # In case there is no file to backup, the backup command will fail and we
    # are ok with that.
    f_backup "/etc/sysconfig/network-scripts/ifcfg-eth0" || true

    f_overwrite_etc_sysconfig_network_script_eth0_v7
    f_increment_completed_step
  else
    echo "exiting"
    exit 0
  fi
}

#------------------------------------------------------------------------------

# Function to recover dracut config file.
f_restore_dracut_config_file() {
  if [ ! -f /etc/dracut.conf.bak ]; then
    echo "/etc/dracut.conf.bak not found. This is an error. Exiting."
    exit 1
  fi

  f_recover "/etc/dracut.conf"
  f_decrement_completed_step
}

#------------------------------------------------------------------------------

# Function to append drivers to dracut config file for rhel6.
f_append_etc_dracut_config_file_v6() {
  params='add_drivers+="hv_vmbus hv_netvsc hv_storvsc"'
  echo $params >> /etc/dracut.conf
}

#------------------------------------------------------------------------------

# Function to add Hyper-V modules to initramfs for rhel6.
f_modify_dracut_config_file_v6() {
  f_interactive_prompt "We will overwrite /etc/dracut.conf. \nAre you sure? (y/n)" "y"
  if [ "$?" -eq 0 ]; then
    f_backup "/etc/dracut.conf"
    f_append_etc_dracut_config_file_v6
    f_increment_completed_step
  else
    echo "exiting"
    exit 0
  fi
}

#------------------------------------------------------------------------------

# Function to append drivers to dracut config file for rhel7.
f_append_etc_dracut_config_file_v7() {
  params='add_drivers+=" hv_vmbus hv_netvsc hv_storvsc "'
  echo $params >> /etc/dracut.conf
}

#------------------------------------------------------------------------------

# Function to add Hyper-V modules to initramfs for rhel7.
f_modify_dracut_config_file_v7() {
  f_interactive_prompt "We will overwrite /etc/dracut.conf. \nAre you sure? (y/n)" "y"
  if [ "$?" -eq 0 ]; then
    f_backup "/etc/dracut.conf"
    f_append_etc_dracut_config_file_v7
    f_increment_completed_step
  else
    echo "exiting"
    exit 0
  fi
}

#------------------------------------------------------------------------------

# Function to add/remove hyperv module to initramfs.
#
# If the first argument ($1) is 'installing' it increments the completed step
# after reconfiguring initramfs by running dracut, else it decrements the
# completed step after running dracut.
f_rebuild_initramfs() {
  local operation="$1"

  # In auto mode we will rebuild initramfs automatically.
  if [ "$is_auto_mode" == true ]; then
    if [ "$operation" == "installing" ]; then
      sudo dracut -f -v
      f_increment_completed_step
    else
      sudo dracut -f
      f_decrement_completed_step
    fi
    return
  fi

  # In non auto mode user will have to manually rebuild the initramfs.
  if [ "$operation" == "installing" ]; then
    while true; do
      echo "Please execute 'sudo dracut -f -v' and enter 'done' when done."
      read op
      if [ "$op" == "done" ]; then
        f_increment_completed_step
        return
      fi
    done
  else
    while true; do
      echo "Please execute 'sudo dracut -f' and enter 'done' when done."
      read op
      if [ "$op" == "done" ]; then
        f_decrement_completed_step
        return
      fi
    done
  fi
}

#------------------------------------------------------------------------------
