#!/bin/bash
#
# Copyright 2018 Cohesity Inc.
#
# Author: Shishir Jindal(sjindal@cohesity.com)
#
# Configures the RHEL 6 VM to be cloned to Azure. The script runs with help
# from our customer. It should be run with sudo privileges.
#
# NOTES:
#   Please read from configure_rhel7_vm.sh.
#
# Output: Exits with 0 upon successful configuration.
#         Exits with non-zero error code upon failure.

# Variable to capture content of /etc/redhat-release.
rhel_release_str=""
version=6

current_dir=$(dirname `readlink -f $0`)
. $current_dir/common_rhel.sh

#------------------------------------------------------------------------------
# Method to install changes needed to clone a RHEL 6 VM to Azure.
#
# The following configuration steps are taken from the Azure documentation
# article at https://docs.microsoft.com/en-us/azure/virtual-machines/linux/
# redhat-create-upload-vhd.
#------------------------------------------------------------------------------

f_execute_install() {
  local completed_step="$1"

  while true; do
    case $completed_step in
      0 )
        f_uninstall_networkmanager_rpm ;;
      1 )
        f_install_etc_sysconfig_network ;;
      2 )
        f_install_etc_sysconfig_network_script_eth0 ;;
      3 )
        f_modify_udev_rules_for_ethernet_interface ;;
      4 )
        f_chkconfig_network_on ;;
      5 )
        f_check_redhat_subscription ;;
      6 )
        f_enable_redhat_extras_repository "$version" ;;
      7 )
        f_modify_boot_line_for_rhel6 ;;
      8 )
        f_modify_dracut_config_file_v6 ;;
      9 )
        f_rebuild_initramfs "installing" ;;
      10 )
        f_enable_ssh_server ;;
      11 )
        f_modify_sshd_config_file ;;
      12 )
        f_install_azure_linux_agent_v6 ;;
      13 )
        f_modify_waagent_conf_file ;;
      14 )
        f_install_cohesity_cloud_migration_flag ;;
      15 )
        echo "Install completed"
        return 0 ;;
    esac
  done
}

#------------------------------------------------------------------------------
# Method to uninstall changes that were done to clone linux VM to Azure.
#------------------------------------------------------------------------------

f_execute_uninstall() {
  local completed_step="$1"

  while true; do
    case $completed_step in
      0 )
        echo "Uninstall completed"
        return 0 ;;
      1 )
        f_install_networkmanager_rpm ;;
      2 )
        f_uninstall_etc_sysconfig_network ;;
      3 )
        f_uninstall_etc_sysconfig_network_script_eth0 ;;
      4 )
        f_restore_udev_rules_for_ethernet_interface ;;
      5 )
        f_restore_chkconfig_network_on ;;
      6 )
        f_disable_redhat_extras_repository "$version" ;;
      7 )
        f_restore_boot_line_for_rhel6 ;;
      8 )
        f_restore_dracut_config_file ;;
      9 )
        f_rebuild_initramfs "uninstalling" ;;
      10 )
        f_disable_ssh_server ;;
      11 )
        f_restore_sshd_config_file ;;
      12 )
        f_uninstall_azure_linux_agent ;;
      13 )
        f_restore_waagent_conf_file ;;
      14 )
        f_disable_waagent_v6 ;;
      15 )
        f_uninstall_cohesity_cloud_migration_flag ;;
    esac
  done
}

#------------------------------------------------------------------------------

# This script works for rhel 6.
rhel_release_file="/etc/redhat-release"
expected_rhel_release_prefix="Red Hat Enterprise Linux Server release 6"

if [ -f $rhel_release_file ]; then
  rhel_release_str=$(cat $rhel_release_file)
fi

if [[ "$rhel_release_str" != "$expected_rhel_release_prefix"* ]]; then
  echo "This file supports '$expected_rhel_release_prefix', found '$rhel_release_str'"
  exit 1
fi

# parse command line parameters for auto and install/uninstall mode.
auto_mode=false
install_mode=false
uninstall_mode=false

while test $# -gt 0
do
    case "$1" in
        --auto) auto_mode=true
            ;;
        --install) install_mode=true
            ;;
        --uninstall) uninstall_mode=true
            ;;
        --*) echo "bad option $1. Exiting..."
              exit 1
            ;;
    esac
    shift
done

f_parse_command_line_params_and_execute $auto_mode $install_mode $uninstall_mode

# At the successful finish we will call sync to flush all the changes we have
# made to the filesystem.
sync
exit 0
